@@ -33,6 +33,7 @@ class DeprecationError(Exception):
 # Ensure we can use `with closing(...) as ... :` syntax
 if getattr(StringIO, '__exit__', False) and \
    getattr(StringIO, '__enter__', False):
+
     def closing(arg):
         return arg
 else:
@@ -62,7 +63,8 @@ CTRLCHR = [r'\r', r'\n', r'\x1b\[A']  # Need to escape [ for regex
 RE_rate = re.compile(r'(\d+\.\d+)it/s')
 RE_ctrlchr = re.compile("(%s)" % '|'.join(CTRLCHR))  # Match control chars
 RE_ctrlchr_excl = re.compile('|'.join(CTRLCHR))  # Match and exclude ctrl chars
-RE_pos = re.compile(r'((\x1b\[A|\r|\n)+((pos\d+) bar:\s+\d+%|\s{3,6})?)')  # NOQA
+RE_pos = re.compile(
+    r'((\x1b\[A|\r|\n)+((pos\d+) bar:\s+\d+%|\s{3,6})?)')  # NOQA
 
 
 class DiscreteTimer(object):
@@ -82,12 +84,13 @@ class DiscreteTimer(object):
 
 class FakeSleep(object):
     '''Wait until the discrete timer reached the required time'''
+
     def __init__(self, dtimer):
         self.dtimer = dtimer
 
     def sleep(self, t):
         end = t + self.dtimer.t
-        while(self.dtimer.t < end):
+        while (self.dtimer.t < end):
             sleep(0.0000001)  # sleep a bit to interrupt (instead of pass)
 
 
@@ -103,8 +106,7 @@ def cpu_timify(t, timer=None):
 
 def pretest():
     # setcheckinterval is deprecated
-    getattr(sys, 'setswitchinterval',
-            getattr(sys, 'setcheckinterval'))(100)
+    getattr(sys, 'setswitchinterval', getattr(sys, 'setcheckinterval'))(100)
 
     if getattr(tqdm, "_instances", False):
         n = len(tqdm._instances)
@@ -233,6 +235,8 @@ def test_format_meter():
 
     assert format_meter(0, 1000, 13) == \
         "  0%|          | 0/1000 [00:13<?, ?it/s]"
+    # If not implementing any changes to _tqdm.py, set prefix='desc'
+    # or else ": : " will be in output, so assertion should change
     assert format_meter(0, 1000, 13, ncols=68, prefix='desc: ') == \
         "desc:   0%|                                | 0/1000 [00:13<?, ?it/s]"
     assert format_meter(231, 1000, 392) == \
@@ -399,8 +403,8 @@ def test_max_interval():
             cpu_timify(t, timer)
 
             # without maxinterval
-            t2 = tqdm(total=total, file=our_file2, miniters=None,
-                      mininterval=0, smoothing=1, maxinterval=None)
+            t2 = tqdm(total=total, file=our_file2, miniters=None, mininterval=0,
+                      smoothing=1, maxinterval=None)
             cpu_timify(t2, timer)
 
             assert t.dynamic_miniters
@@ -818,8 +822,8 @@ def test_close():
         exres = res + '\n'
         if exres != our_file.read():
             our_file.seek(0)
-            raise AssertionError("\nExpected:\n{0}\nGot:{1}\n".format(
-                exres, our_file.read()))
+            raise AssertionError(
+                "\nExpected:\n{0}\nGot:{1}\n".format(exres, our_file.read()))
 
     # Closing after the output stream has closed
     with closing(StringIO()) as our_file:
@@ -997,8 +1001,8 @@ def test_position():
     # Artificially test nested loop printing
     # Without leave
     our_file = StringIO()
-    t = tqdm(total=2, file=our_file, miniters=1, mininterval=0,
-             maxinterval=0, desc='pos2 bar', leave=False, position=2)
+    t = tqdm(total=2, file=our_file, miniters=1, mininterval=0, maxinterval=0,
+             desc='pos2 bar', leave=False, position=2)
     t.update()
     t.close()
     our_file.seek(0)
@@ -1014,8 +1018,8 @@ def test_position():
 
     # Test iteration-based tqdm positioning
     our_file = StringIO()
-    for _ in trange(2, file=our_file, miniters=1, mininterval=0,
-                    maxinterval=0, desc='pos0 bar', position=0):
+    for _ in trange(2, file=our_file, miniters=1, mininterval=0, maxinterval=0,
+                    desc='pos0 bar', position=0):
         for _ in trange(2, file=our_file, miniters=1, mininterval=0,
                         maxinterval=0, desc='pos1 bar', position=1):
             for _ in trange(2, file=our_file, miniters=1, mininterval=0,
@@ -1052,12 +1056,12 @@ def test_position():
 
     # Test manual tqdm positioning
     our_file = StringIO()
-    t1 = tqdm(total=2, file=our_file, miniters=1, mininterval=0,
-              maxinterval=0, desc='pos0 bar', position=0)
-    t2 = tqdm(total=2, file=our_file, miniters=1, mininterval=0,
-              maxinterval=0, desc='pos1 bar', position=1)
-    t3 = tqdm(total=2, file=our_file, miniters=1, mininterval=0,
-              maxinterval=0, desc='pos2 bar', position=2)
+    t1 = tqdm(total=2, file=our_file, miniters=1, mininterval=0, maxinterval=0,
+              desc='pos0 bar', position=0)
+    t2 = tqdm(total=2, file=our_file, miniters=1, mininterval=0, maxinterval=0,
+              desc='pos1 bar', position=1)
+    t3 = tqdm(total=2, file=our_file, miniters=1, mininterval=0, maxinterval=0,
+              desc='pos2 bar', position=2)
     for _ in _range(2):
         t1.update()
         t3.update()
@@ -1095,8 +1099,7 @@ def test_position():
                  '\x1b[A\x1b[A']
         if res != exres:
             raise AssertionError(
-                "\nExpected:\n{0}\nGot:\n{1}\n".format(
-                    str(exres), str(res)))
+                "\nExpected:\n{0}\nGot:\n{1}\n".format(str(exres), str(res)))
 
         t2.close()
         t4 = tqdm(total=10, file=our_file, desc='pos3 bar', mininterval=0)
@@ -1114,8 +1117,7 @@ def test_position():
                  '\x1b[A\x1b[A']
         if res != exres:
             raise AssertionError(
-                "\nExpected:\n{0}\nGot:\n{1}\n".format(
-                    str(exres), str(res)))
+                "\nExpected:\n{0}\nGot:\n{1}\n".format(str(exres), str(res)))
         t4.close()
         t3.close()
         t1.close()
@@ -1123,14 +1125,16 @@ def test_position():
 
 @with_setup(pretest, posttest)
 def test_set_description():
-    """Test set description"""
-    with closing(StringIO()) as our_file:
-        with tqdm(desc='Hello', file=our_file) as t:
-            assert t.desc == 'Hello: '
-            t.set_description('World')
-            assert t.desc == 'World: '
-            t.set_description()
-            assert t.desc == ''
+   """Test set description"""
+   with closing(StringIO()) as our_file:
+       with tqdm(desc='Hello', file=our_file) as t:
+           assert t.desc == 'Hello'
+           t.set_description_str('World')
+           assert t.desc == 'World'
+           t.set_description()
+           assert t.desc == ''
+           t.set_description('Bye')
+           assert t.desc == 'Bye: '
 
 
 @with_setup(pretest, posttest)
@@ -1154,8 +1158,7 @@ def test_deprecated_gui():
             # t.close()
             # len(tqdm._instances) += 1  # undo the close() decrement
 
-        t = tqdm(_range(3), gui=True, file=our_file,
-                 miniters=1, mininterval=0)
+        t = tqdm(_range(3), gui=True, file=our_file, miniters=1, mininterval=0)
         try:
             for _ in t:
                 pass
@@ -1218,8 +1221,7 @@ def test_clear():
     with closing(StringIO()) as our_file:
         t1 = tqdm(total=10, file=our_file, desc='pos0 bar',
                   bar_format='{l_bar}')
-        t2 = trange(10, file=our_file, desc='pos1 bar',
-                    bar_format='{l_bar}')
+        t2 = trange(10, file=our_file, desc='pos1 bar', bar_format='{l_bar}')
         before = squash_ctrlchars(our_file.getvalue())
         t2.clear()
         t1.clear()
@@ -1392,9 +1394,8 @@ def test_autodisable_enable():
 def test_deprecation_exception():
     def test_TqdmDeprecationWarning():
         with closing(StringIO()) as our_file:
-            raise (TqdmDeprecationWarning('Test!',
-                                          fp_write=getattr(our_file, 'write',
-                                                           sys.stderr.write)))
+            raise (TqdmDeprecationWarning('Test!', fp_write=getattr(
+                our_file, 'write', sys.stderr.write)))
 
     def test_TqdmDeprecationWarning_nofpwrite():
         raise (TqdmDeprecationWarning('Test!', fp_write=None))
@@ -1443,8 +1444,8 @@ def test_monitoring_thread():
     # Set monitor interval
     tqdm.monitor_interval = maxinterval
     with closing(StringIO()) as our_file:
-        with tqdm(total=total, file=our_file, miniters=500,
-                  mininterval=0.1, maxinterval=maxinterval) as t:
+        with tqdm(total=total, file=our_file, miniters=500, mininterval=0.1,
+                  maxinterval=maxinterval) as t:
             cpu_timify(t, timer)
             # Do a lot of iterations in a small timeframe
             # (smaller than monitor interval)
@@ -1489,11 +1490,11 @@ def test_monitoring_thread():
     TMonitor._time = timer.time
     TMonitor._sleep = sleeper.sleep
     with closing(StringIO()) as our_file:
-        with tqdm(total=total, file=our_file, miniters=500,
-                  mininterval=0.1, maxinterval=maxinterval) as t1:
+        with tqdm(total=total, file=our_file, miniters=500, mininterval=0.1,
+                  maxinterval=maxinterval) as t1:
             # Set high maxinterval for t2 so monitor does not need to adjust it
-            with tqdm(total=total, file=our_file, miniters=500,
-                      mininterval=0.1, maxinterval=1E5) as t2:
+            with tqdm(total=total, file=our_file, miniters=500, mininterval=0.1,
+                      maxinterval=1E5) as t2:
                 cpu_timify(t1, timer)
                 cpu_timify(t2, timer)
                 # Do a lot of iterations in a small timeframe
@@ -1533,8 +1534,8 @@ def test_postfix():
 
     # Test postfix set after init
     with closing(StringIO()) as our_file:
-        with trange(10, file=our_file, desc='pos1 bar',
-                    bar_format='{r_bar}', postfix=None) as t2:
+        with trange(10, file=our_file, desc='pos1 bar', bar_format='{r_bar}',
+                    postfix=None) as t2:
             t2.set_postfix(**postfix)
             t2.refresh()
             out2 = our_file.getvalue()
@@ -1546,8 +1547,8 @@ def test_postfix():
 
     # Test postfix set after init and with ordered dict
     with closing(StringIO()) as our_file:
-        with trange(10, file=our_file, desc='pos2 bar',
-                    bar_format='{r_bar}', postfix=None) as t3:
+        with trange(10, file=our_file, desc='pos2 bar', bar_format='{r_bar}',
+                    postfix=None) as t3:
             t3.set_postfix(postfix_order, **postfix)
             t3.refresh()
             out3 = our_file.getvalue()
@@ -1555,6 +1556,17 @@ def test_postfix():
     out3 = out3[1:-1].split(', ')[3:]
     assert out3 == expected_order
 
+    # Test setting postfix string directly
+    with closing(StringIO()) as our_file:
+        with trange(10, file=our_file, desc='pos2 bar', bar_format='{r_bar}',
+                    postfix=None) as t4:
+            t4.set_postfix_str("Hello")
+            t4.refresh()
+            out4 = our_file.getvalue()
+
+    out4 = out4[1:-1].split(', ')[3:]
+    assert out4 == ["Hello"]
+
 
 class DummyTqdmFile(object):
     """Dummy file-like that will write to tqdm"""
